@@ -4,5 +4,6 @@
 
 from .connections import (
     getUserSession, getPostInfo, getOfficialVideoPost, getInkeyData,
-    getLivePlayInfo, getLiveStatus, getVodPlayInfo
+    getLivePlayInfo, getLiveStatus, getVodPlayInfo, getFVideoInkeyData,
+    getPostComments, getPostStarComments, getFVideoPlayInfo
 )
