@@ -6,7 +6,12 @@ from . import variables as gv
 from .exception import (
     auto_raise, APINetworkError, APISignInFailedError, APIJSONParesError
 )
-from .parser import parseVideoSeqFromPostInfo, sessionUserCheck, parseVodIdFromOffcialVideoPost, parseUpcomingFromPage
+from .parser import (
+    parseVideoSeqFromPostInfo, sessionUserCheck, parseVodIdFromOffcialVideoPost,
+    parseUpcomingFromPage, response_json_stripper, comment_parser, CommentItem,
+    next_page_checker
+)
+from typing import Generator
 
 
 def getUserSession(email, pwd, silent=False):
@@ -20,9 +25,8 @@ def getUserSession(email, pwd, silent=False):
     """
 
     # Make request
-    data = {'email': email, 'pwd': pwd}
-    headers = {**gv.HeaderCommon, **gv.APISignInReferer}
-    sr = reqWrapper.post(gv.APISignInUrl, data=data, headers=headers, wait=0.5, status=[200])
+    sr = reqWrapper.post(**gv.endpoint_auth(email, pwd),
+                         wait=0.5, status=[200])
 
     if sr.success:
         # Case <Sign-in Failed (Exception)>
@@ -46,25 +50,39 @@ def getInkeyData(videoSeq, session=None, silent=False):
 
     :param videoSeq: postId from VLIVE (like ######)(Numbers)
     :param session: use specific session
-    :type session: reqWrapper.requests.Session
     :param silent: Return `None` instead of Exception
     :return: Inkey data
     :rtype: dict
     """
 
     # Make request
-    headers = {**gv.HeaderCommon, **gv.APIofficialVideoPostReferer(videoSeq)}
-    sr = reqWrapper.get(gv.APIInkeyUrl(videoSeq), headers=headers, wait=0.5, session=session, status=[200])
+    sr = reqWrapper.get(**gv.endpoint_vod_inkey(videoSeq),
+                        wait=0.5, session=session, status=[200])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
+    else:
+        auto_raise(APINetworkError, silent)
+
+    return None
+
+
+def getFVideoInkeyData(fvideo, session=None, silent=False):
+    r""" get Inkey Data
+
+    :param fvideo: file video id from VLIVE (like ######)(Numbers)
+    :param session: use specific session
+    :param silent: Return `None` instead of Exception
+    :return: Inkey data
+    :rtype: dict
+    """
+
+    # Make request
+    sr = reqWrapper.get(**gv.endpoint_fvideo_inkey(fvideo),
+                        wait=0.5, session=session, status=[200])
+
+    if sr.success:
+        return response_json_stripper(sr.response.json(), silent=silent)['inKey']
     else:
         auto_raise(APINetworkError, silent)
 
@@ -96,25 +114,16 @@ def getPostInfo(post, session=None, silent=False):
 
     :param post: postId from VLIVE (like #-########)
     :param session: use specific session
-    :type session: reqWrapper.requests.Session
     :param silent: Return `None` instead of Exception
     :return: videoInfo
     :rtype: dict
     """
 
-    # Make request
-    headers = {**gv.HeaderCommon, **gv.APIPostReferer(post)}
-    sr = reqWrapper.get(gv.APIPostUrl(post), headers=headers, wait=0.5, session=session, status=[200, 403])
+    sr = reqWrapper.get(**gv.endpoint_post(post),
+                        wait=0.5, session=session, status=[200, 403])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
     else:
         auto_raise(APINetworkError, silent)
 
@@ -131,19 +140,11 @@ def getOfficialVideoPost(videoSeq, session=None, silent=False):
     :rtype: dict
     """
 
-    headers = {**gv.HeaderCommon, **gv.APIofficialVideoPostReferer(videoSeq)}
-    sr = reqWrapper.get(gv.APIofficialVideoPostUrl(videoSeq), headers=headers,
+    sr = reqWrapper.get(**gv.endpoint_official_video_post(videoSeq),
                         session=session, wait=0.5, status=[200, 403])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
     else:
         auto_raise(APINetworkError, silent)
 
@@ -155,7 +156,6 @@ def getLivePlayInfo(videoSeq, session=None, vpdid2=None, silent=False):
 
     :param videoSeq: postId from VLIVE (like ######)(Numbers)
     :param session: use specific session
-    :type session: reqWrapper.requests.Session
     :param vpdid2: vpdid2 from api.getInkeyData()
     :param silent: Return `None` instead of Exception
     :return: `LivePlayInfoV3` Data
@@ -167,24 +167,12 @@ def getLivePlayInfo(videoSeq, session=None, vpdid2=None, silent=False):
         if sessionUserCheck(session):
             vpdid2 = getVpdid2(session, silent=silent)
 
-    # Add vpdid2 param, if vpdid2 is valid
-    url = gv.APILiveV3PlayInfoUrl(videoSeq)
-    if vpdid2 is not None:
-        url += "&vpdid2=%s" % vpdid2
-
     # Make request
-    headers = {**gv.HeaderCommon, **gv.APIofficialVideoPostReferer(videoSeq)}
-    sr = reqWrapper.get(url, headers=headers, session=session, status=[200, 403])
+    sr = reqWrapper.get(**gv.endpoint_live_play_info(videoSeq, vpdid2),
+                        session=session, status=[200, 403])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
     else:
         auto_raise(APINetworkError, silent)
 
@@ -201,18 +189,11 @@ def getLiveStatus(videoSeq, silent=False):
     """
 
     # Make request
-    headers = {**gv.HeaderCommon, **gv.APIofficialVideoPostReferer(videoSeq)}
-    sr = reqWrapper.get(gv.APILiveV2StatusUrl(videoSeq), headers=headers, wait=0.2, status=[200])
+    sr = reqWrapper.get(**gv.endpoint_live_status(videoSeq),
+                        wait=0.2, status=[200])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
     else:
         auto_raise(APINetworkError, silent)
 
@@ -242,46 +223,144 @@ def getVodPlayInfo(videoSeq, vodId=None, session=None, silent=False):
         vodId = getVodId(videoSeq)
 
     # make request
-    url = gv.APIVodPlayInfoUrl(vodId, inkey)
-    headers = {**gv.HeaderCommon, **gv.APIVodPlayInfoReferer}
-    sr = reqWrapper.get(url, headers=headers, session=session, wait=0.3, status=[200, 403])
+    sr = reqWrapper.get(**gv.endpoint_vod_play_info(vodId, inkey),
+                        session=session, wait=0.3, status=[200, 403])
 
     if sr.success:
-        parsed_json = sr.response.json()
-        if "code" in parsed_json:
-            if "result" in parsed_json:
-                return parsed_json['result']
-            else:
-                return None
-        else:
-            return parsed_json
+        return response_json_stripper(sr.response.json(), silent=silent)
     else:
         auto_raise(APINetworkError, silent=silent)
 
 
-def loadUpcoming(day=None, silent=False):
+def getFVideoPlayInfo(videoSeqId, videoVodId, session=None, silent=False):
+    r""" Get fvideo VOD Data
+
+    :param videoSeqId: videoId from attachment-id (like #-########)
+    :param videoVodId: videoId from attachment/uploadInfo (like #-########)
+    :param session: use specific session
+    :param silent: Return `None` instead of Exception
+    :return: 
+    """
+
+    inkey = getFVideoInkeyData(fvideo=videoSeqId, session=session)
+    sr = reqWrapper.get(**gv.endpoint_vod_play_info(videoVodId, inkey),
+                        session=session, wait=0.3, status=[200, 403])
+
+    if sr.success:
+        return response_json_stripper(sr.response.json(), silent=silent)
+    else:
+        auto_raise(APINetworkError, silent=silent)
+
+
+def getUpcomingList(date=None, silent=False):
     params = dict()
-    if day is not None:
-        params.update({"d": day})
+    if date is not None:
+        params.update({"d": date})
 
     # make request
     url = "https://www.vlive.tv/upcoming"
-    headers = gv.HeaderCommon
-    sr = reqWrapper.get(url, params=params, headers=headers)
+    sr = reqWrapper.get(url, params=params, headers=gv.HeaderCommon)
+
     if sr.success:
-        return sr.response.text
+        return parseUpcomingFromPage(sr.response.text)
     else:
         auto_raise(APINetworkError, silent=silent)
 
     return None
 
 
-def getUpcomingList(date=None, silent=False):
-    html = loadUpcoming(day=date, silent=silent)
-    if html is None:
-        return None
+def getPostComments(post, session=None, after=None, silent=False):
+    r""" Get post's comments
+
+    :param post: postId from VLIVE (like #-########)
+    :param after: load page after #comment-Id
+    :param session: use specific session
+    :param silent: Return `None` instead of Exception
+    :return: comments
+    :rtype: dict
+    """
+
+    # Make request
+    sr = reqWrapper.get(**gv.endpoint_post_comments(post, after),
+                        wait=0.5, session=session, status=[200, 403])
+
+    if sr.success:
+        stripped_data = response_json_stripper(sr.response.json(), silent=silent)
+        if 'data' in stripped_data:
+            stripped_data['data'] = comment_parser(stripped_data['data'])
+        return stripped_data
     else:
-        return parseUpcomingFromPage(html)
+        auto_raise(APINetworkError, silent)
+
+    return None
+
+
+def getPostCommentsIter(post, session=None):
+    r""" Get post's comments
+
+    :param post: postId from VLIVE (like #-########)
+    :param session: use specific session
+    :return: comments generator
+    :rtype: Generator[CommentItem, None, None]
+    """
+
+    data = getPostComments(post, session=session)
+    after = next_page_checker(data)
+    for item in data['data']:
+        yield item
+
+    while after:
+        data = getPostComments(post, session=session, after=after)
+        after = next_page_checker(data)
+        for item in data['data']:
+            yield item
+
+
+def getPostStarComments(post, session=None, after=None, silent=False):
+    r""" Get post's star comments
+
+    :param post: postId from VLIVE (like #-########)
+    :param after: load page after #comment-Id
+    :param session: use specific session
+    :param silent: Return `None` instead of Exception
+    :return: comments
+    :rtype: dict
+    """
+
+    # Make request
+    sr = reqWrapper.get(**gv.endpoint_post_star_comments(post, after),
+                        wait=0.5, session=session, status=[200, 403])
+
+    if sr.success:
+        stripped_data = response_json_stripper(sr.response.json(), silent=silent)
+        if 'data' in stripped_data:
+            stripped_data['data'] = comment_parser(stripped_data['data'])
+        return stripped_data
+    else:
+        auto_raise(APINetworkError, silent)
+
+    return None
+
+
+def getPostStarCommentsIter(post, session=None):
+    r""" Get post's star comments as Iterable
+
+    :param post: postId from VLIVE (like #-########)
+    :param session: use specific session
+    :return: comments generator
+    :rtype: Generator[CommentItem, None, None]
+    """
+
+    data = getPostStarComments(post, session=session)
+    after = next_page_checker(data)
+    for item in data['data']:
+        yield item
+
+    while after:
+        data = getPostStarComments(post, session=session, after=after)
+        after = next_page_checker(data)
+        for item in data['data']:
+            yield item
 
 
 def postIdToVideoSeq(post, silent=False):
@@ -293,6 +372,21 @@ def postIdToVideoSeq(post, silent=False):
     :rtype: str
     """
 
-    postInfo = getPostInfo(post, silent=silent)
+    postInfo = getPostInfo(post, silent=True)
 
     return parseVideoSeqFromPostInfo(postInfo, silent=silent)
+
+
+def postTypeDetector(post, silent=False):
+    r""" Check given postId is Post or Video
+
+    :param post: postId from VLIVE (like #-########)
+    :param silent: Return `None` instead of Exception
+    :return: str "POST" or "VIDEO"
+    :rtype: str
+    """
+    data = getPostInfo(post, silent=silent)
+    if data is not None:
+        return data['contentType']
+
+    return None
