@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 # auto raise exception
 def auto_raise(exception, silent):
     if not silent:
@@ -22,9 +23,17 @@ class APISignInFailedError(APIError):
     """ Failed to Sign in """
 
 
+class APIServerResponseError(APIError):
+    """ Warning if server response only error"""
+
+
 class ModelError(Exception):
     """ Common Model Error """
 
 
 class ModelInitError(Exception):
     """ Model Initialize Error """
+
+
+class APIServerResponseWarning(Warning):
+    """ Warning if server response with error"""
