@@ -1,9 +1,13 @@
 # -*- coding: utf-8 -*-
-from .exception import auto_raise, APIJSONParesError
+from warnings import warn
+from .exception import auto_raise, APIJSONParesError, APIServerResponseWarning, APIServerResponseError
 from collections import namedtuple
 from bs4 import BeautifulSoup
+from datetime import datetime
 
 UpcomingVideo = namedtuple("UpcomingVideo", "seq time cseq cname ctype name type product")
+CommentItem = namedtuple("CommentItem", 'commentId author body sticker createdAt '
+                                        'commentCount emotionCount isRestricted parent root')
 
 
 def parseVideoSeqFromPostInfo(info, silent=False):
@@ -116,3 +120,61 @@ def parseVodIdFromOffcialVideoPost(post, silent=False):
         auto_raise(APIJSONParesError("Given data is post data"), silent=silent)
 
     return None
+
+
+def response_json_stripper(parsed_json_dict: dict, silent=False):
+    # if data has success code
+    if "code" in parsed_json_dict:
+        if "result" in parsed_json_dict:
+            parsed_json_dict = parsed_json_dict['result']
+        else:
+            parsed_json_dict = None
+    elif 'errorCode' in parsed_json_dict:
+        err_tuple = (parsed_json_dict['errorCode'], parsed_json_dict['message'].replace("\n", " "))
+        if 'data' in parsed_json_dict:
+            warn("Response has error [%s] %s" % err_tuple, APIServerResponseWarning)
+            parsed_json_dict = parsed_json_dict['data']
+        else:
+            auto_raise(APIServerResponseError('[%s] %s' % err_tuple), silent=silent)
+
+    # if data has data field (Fanship)
+    if "data" in parsed_json_dict and len(parsed_json_dict) == 1:
+        parsed_json_dict = parsed_json_dict["data"]
+
+    return parsed_json_dict
+
+
+def next_page_checker(page):
+    if 'nextParams' in page['paging']:
+        return page['paging']['nextParams']['after']
+    else:
+        return None
+
+
+def comment_parser(comment_list: list):
+    n_list = []
+    for comment in comment_list:
+        n_list.append(CommentItem(
+            commentId=comment['commentId'],
+            author=comment['author'],
+            body=comment['body'],
+            sticker=comment['sticker'],
+            createdAt=comment['createdAt'],
+            commentCount=comment['commentCount'],
+            emotionCount=comment['emotionCount'],
+            isRestricted=comment['isRestricted'],
+            parent=comment['parent'],
+            root=comment['root']
+        ))
+
+    return n_list
+
+
+def max_res_from_play_info(play_info):
+    vl = play_info['videos']['list']
+    sorted_res = sorted(vl, key=lambda x: x['bitrate']['video'], reverse=True)
+    return sorted_res[0]
+
+
+def format_epoch(epoch, fmt):
+    return datetime.fromtimestamp(epoch).strftime(fmt)
\ No newline at end of file
